"""
    This file has a garbage version string in it 
    intentionally so that the build.sh script at
    the top of this repository can create the
    version string according to calver.org 
    convention.
"""
__version__ = '0.0.0'
__version__ = '18.8.10'
